
//Define the Main Clock setup options. Done here for the delay function setup
#define DCO_RANGE_SEL DCORSEL
#define DCO_FREQ_SEL DCOFSEL_4
#define F_CPU_SCALE  DIVM_0

/**
 * @defgroup Delay_function
 * @brief Directives to create the _delay_us macro.
 * @{
 */
/**
 * @def _delay_us(__us)
 * @brief Delay for a specified time using the __delay_cycles primitive.
 *
 * @param[in] __us - time to delay for
 * @return none
 * @note Assumes using the internal DCO. Will need modification to work with
 *       XTAL. VLO is two innacurate (6%) for this to be a good delay anyway.
 *       Max possible delay is (2**32-1)*F_CPU
 *       That would be 178s for 24MHz.
 *       helped by https://docs.microsoft.com/en-us/cpp/preprocessor/hash-if-hash-elif-hash-else-and-hash-endif-directives-c-cpp?view=vs-2019
 *       and https://github.com/ab2tech/msp430/blob/master/include/msp/delay.h
 *
 */
#if DCO_RANGE_SEL==DCORSEL
	#if DCO_FREQ_SEL>DCOFSEL_3
		#define F_CPU_RANGE 5333000UL
	#else
		#define F_CPU_RANGE 5333000UL*1.5
	#endif
#else
	#define F_CPU_RANGE 2667000UL
#endif

#if DCO_FREQ_SEL==DCOFSEL_1
  #define F_CPU F_CPU_RANGE*1.0
#elif DCO_FREQ_SEL==DCOFSEL_2
  #define F_CPU F_CPU_RANGE*1.313
#elif DCO_FREQ_SEL==DCOFSEL_3
  #define F_CPU F_CPU_RANGE*1.5
#elif DCO_FREQ_SEL==DCOFSEL_4
  #define F_CPU F_CPU_RANGE*2.0
#elif DCO_FREQ_SEL==DCOFSEL_5
  #define F_CPU F_CPU_RANGE*2.62
#elif DCO_FREQ_SEL==DCOFSEL_6
  #define F_CPU F_CPU_RANGE*3.0
#else //DCO_FREQ_SEL==DCOFSEL_0
  #define F_CPU 1000000UL
#endif

#define F_CPU_SCALED_US F_CPU/(1<<F_CPU_SCALE)/1000000.0
            /* Number of CPU cycles per us */

#define _delay_us(__us) \
  if((uint32_t) (F_CPU_SCALED_US * __us) != F_CPU_SCALED_US * __us)\
        __delay_cycles((uint32_t) ( F_CPU_SCALED_US * __us)+1);\
  else __delay_cycles((uint32_t) ( F_CPU_SCALED_US * __us))
/** @} */
